@@ -1,9 +1,13 @@
 from datetime import datetime, date
+from findertools import select
 from uuid import uuid4
+from arangodb.api import Collection, Client
 
 
 class ModelField(object):
 
+    is_saved_in_model = True
+
     class NotNullableFieldException(Exception):
         """
             Field cannot be null
@@ -37,7 +41,25 @@ class ModelField(object):
 
         pass
 
-    def on_create(self):
+    def on_init(self, model_class):
+        """
+        """
+
+        pass
+
+    def on_destroy(self, model_class):
+        """
+        """
+
+        pass
+
+    def on_create(self, model_instance):
+        """
+        """
+
+        self.model_instance = model_instance
+
+    def on_save(self, model_instance):
         """
         """
 
@@ -73,6 +95,7 @@ class ModelField(object):
 
         return self.dumps()
 
+
 class BooleanField(ModelField):
 
     def __init__(self, **kwargs):
@@ -137,6 +160,7 @@ class BooleanField(ModelField):
         else:
             return False
 
+
 class CharField(ModelField):
 
     class TooLongStringException(Exception):
@@ -223,43 +247,13 @@ class UuidField(CharField):
 
         self.auto_create = auto_create
 
-    def dumps(self):
-        """
-        """
-
-        return self.text
-
-    def loads(self, string_val):
-        """
-        """
-
-        self.text = string_val
-
-    def on_create(self):
+    def on_create(self, model_instance):
         """
         """
 
         if self.auto_create and self.text is None or self.text == '':
             self.text = str(uuid4())
 
-    def set(self, *args, **kwargs):
-        """
-        """
-
-        if len(args) is 1:
-            text = args[0]
-
-            if isinstance(text, basestring):
-                self.text = u'%s' % args[0]
-            else:
-                raise UuidField.WrongInputTypeException()
-
-    def get(self):
-        """
-        """
-
-        return self.text
-
 
 class NumberField(ModelField):
 
@@ -516,5 +510,113 @@ class ForeignKeyField(ModelField):
 
         if super(ForeignKeyField, self).__eq__(other):
             return self.relation_model == other.relation_model
+        else:
+            return False
+
+
+class ManyToManyField(ModelField):
+
+    is_saved_in_model = False
+
+    def __init__(self, to, related_name, **kwargs):
+        """
+        """
+
+        super(ManyToManyField, self).__init__(**kwargs)
+
+        self.relation_class = to
+        self.related_name = related_name
+        self.relation_collection = None
+
+        # Data
+        self.unsaved_data = False
+        self.related_queryset = None
+
+    def on_init(self, model_class):
+        """
+        """
+
+        if not self.related_name is None:
+            relation_name = self._get_relation_collection_name(model_class)
+
+            try:
+                self.relation_collection = Collection.create(name=relation_name, database=Client.instance().database, type=3)
+            except:
+                self.relation_collection = Collection.get_loaded_collection(name=relation_name)
+
+            fields = self.relation_class._model_meta_data._fields
+            otherside_field = ManyToManyField(to=model_class, related_name=None)
+            fields[self.related_name] = otherside_field
+
+            # Configure other side field
+            otherside_field.related_queryset = self.relation_class.objects.all()
+            otherside_field.relation_collection = self.relation_collection
+
+            self.related_queryset = self.relation_class.objects.all()
+
+    def on_destroy(self, model_class):
+        """
+        """
+
+        if not self.related_name is None:
+            relation_name = self._get_relation_collection_name(model_class)
+            Collection.remove(name=relation_name)
+
+    def on_save(self, model_instance):
+        """
+        """
+
+        if self.unsaved_data:
+            new_models = self.related_queryset._cache
+
+            for model in new_models:
+                related_model_document = model.document
+                model_document = model_instance.document
+
+                # Create relation
+                self.relation_collection.create_edge(from_doc=model_document, to_doc=related_model_document)
+
+    def _get_relation_collection_name(self, model_class):
+        """
+        """
+
+        return 'relation_%s_%s' % ( model_class.get_collection_name(), self.relation_class.get_collection_name() )
+
+    def set(self, *args, **kwargs):
+        """
+        """
+
+        if len(args) is 1:
+            related_models = args[0]
+            self.related_queryset._cache = related_models
+            self.unsaved_data = True
+
+    def get(self):
+        """
+        """
+
+        if self.unsaved_data is True:
+            return self.related_queryset
+        else:
+
+            if self.related_name is None:
+                return self.related_queryset.get_field_relations(
+                    end_model=self.model_instance,
+                    relation_collection=self.relation_collection.name,
+                    related_model_class=self.relation_class
+                )
+            else:
+                return self.related_queryset.get_field_relations(
+                    start_model=self.model_instance,
+                    relation_collection=self.relation_collection.name,
+                    related_model_class=self.relation_class
+                )
+
+    def __eq__(self, other):
+        """
+        """
+
+        if super(ManyToManyField, self).__eq__(other):
+            return self.related_queryset == other.related_queryset
         else:
             return False
\ No newline at end of file
