@@ -7,11 +7,11 @@ from arangodb.index.api import Index
 from arangodb.index.general import FulltextIndex, CapConstraintIndex
 from arangodb.index.unique import HashIndex, SkiplistIndex, GeoIndex
 from arangodb.orm.fields import CharField, ForeignKeyField, NumberField, DatetimeField, DateField, BooleanField, \
-    UuidField
+    UuidField, ManyToManyField
 from arangodb.orm.models import CollectionModel
 from arangodb.query.advanced import Query, Traveser
 from arangodb.query.utils.document import create_document_from_result_dict
-from arangodb.query.simple import SimpleQuery
+from arangodb.query.simple import SimpleQuery, SimpleIndexQuery
 from arangodb.transaction.controller import Transaction, TransactionController
 from arangodb.user import User
 
@@ -334,6 +334,50 @@ class SimpleQueryTestCase(ExtendedTestCase):
         self.assertDocumentsEqual(doc, self.col1_doc1)
 
 
+class SimpleIndexQueryTestCase(ExtendedTestCase):
+    def setUp(self):
+        self.database_name = 'testcase_simple_index_query_123'
+        self.db = Database.create(name=self.database_name)
+
+        # Create test data
+        self.test_1_col = self.db.create_collection('foo_1')
+
+        self.hash_index = Index(self.test_1_col, HashIndex(fields=[
+            'username'
+        ]))
+
+        self.hash_index.save()
+
+        self.col1_doc1 = self.test_1_col.create_document()
+        self.col1_doc1.username='surgent'
+        self.col1_doc1.bla='aaa'
+        self.col1_doc1.save()
+
+        self.col1_doc2 = self.test_1_col.create_document()
+        self.col1_doc2.username='name killer'
+        self.col1_doc2.bla='xxx'
+        self.col1_doc2.save()
+
+    def tearDown(self):
+        # Delete index
+        self.hash_index.delete()
+
+        # They need to be deleted
+        Collection.remove(name=self.test_1_col.name)
+
+        Database.remove(name=self.database_name)
+
+    def test_get_only_by_hash_index(self):
+        """
+        """
+
+        doc1 = SimpleIndexQuery.get_by_example(collection=self.test_1_col, example_data={
+            'username': self.col1_doc1.username
+        })
+
+        self.assertDocumentsEqual(doc1, self.col1_doc1)
+
+
 class TraveserTestCase(ExtendedTestCase):
     def setUp(self):
         self.database_name = 'testcase_traverser_query_123'
@@ -753,7 +797,7 @@ class UuidFieldTestCase(unittest.TestCase):
 
         self.assertEqual(uuid.text, None)
 
-        uuid.on_create()
+        uuid.on_create(model_instance=None)
 
         self.assertTrue(isinstance(uuid.text, basestring))
 
@@ -894,6 +938,126 @@ class ForeignkeyFieldTestCase(unittest.TestCase):
         self.assertEqual(field1, field2)
 
 
+class ManyToManyFieldTestCase(unittest.TestCase):
+
+    def setUp(self):
+        self.database_name = 'only_many_to_many_field_123'
+        self.db = Database.create(name=self.database_name)
+
+    def tearDown(self):
+        Database.remove(name=self.database_name)
+
+    def test_basic_creation_with_default(self):
+
+
+        class EndModel(CollectionModel):
+
+            test_field = CharField()
+
+
+        class StartModel(CollectionModel):
+            collection_name = 'never_to_be_seen_again'
+
+            others = ManyToManyField(to=EndModel, related_name='starters')
+
+        EndModel.init()
+        StartModel.init()
+
+        end_model1 = EndModel()
+        end_model1.test_field = 'foo'
+        end_model1.save()
+
+        end_model2 = EndModel()
+        end_model2.test_field = 'bar'
+        end_model2.save()
+
+        start_model = StartModel()
+        start_model.others = [end_model1, end_model2]
+        start_model.save()
+
+        relation_collection_name = start_model.get_field(name='others')._get_relation_collection_name(StartModel)
+        col = Collection.get_loaded_collection(name=relation_collection_name)
+        relation_documents = col.documents()
+
+        self.assertEqual(len(relation_documents), 2)
+
+        rel1 = relation_documents[0]
+        rel2 = relation_documents[1]
+
+        # From is always the same
+        self.assertEqual(rel1._from, start_model.document.id)
+        self.assertEqual(rel2._from, start_model.document.id)
+
+        is_first_the_first_end_model = rel1._to == end_model1.document.id
+        is_first_the_second_end_model = rel1._to == end_model2.document.id
+        self.assertTrue(is_first_the_first_end_model or is_first_the_second_end_model)
+
+        is_second_the_first_end_model = rel2._to == end_model1.document.id
+        is_second_the_second_end_model = rel2._to == end_model2.document.id
+        self.assertTrue(is_second_the_first_end_model or is_second_the_second_end_model)
+
+        StartModel.destroy()
+        EndModel.destroy()
+
+    def test_getting_related_objects(self):
+
+
+        class EndModel(CollectionModel):
+
+            test_field = CharField()
+
+
+        class StartModel(CollectionModel):
+            collection_name = 'never_to_be_seen_again'
+
+            others = ManyToManyField(to=EndModel, related_name='starters')
+
+        EndModel.init()
+        StartModel.init()
+
+        end_model1 = EndModel()
+        end_model1.test_field = 'foo'
+        end_model1.save()
+
+        end_model2 = EndModel()
+        end_model2.test_field = 'bar'
+        end_model2.save()
+
+        end_model3 = EndModel()
+        end_model3.test_field = 'extra'
+        end_model3.save()
+
+        start_model = StartModel()
+        start_model.others = [end_model1, end_model2]
+        start_model.save()
+
+        start_model = StartModel.objects.get(_id=start_model.document.id)
+
+        related_models = start_model.others
+
+        self.assertEqual(len(related_models), 2)
+
+        rel1 = related_models[0]
+        rel2 = related_models[1]
+
+        is_first_the_first_end_model = rel1.document.id == end_model1.document.id
+        is_first_the_second_end_model = rel1.document.id == end_model2.document.id
+        self.assertTrue(is_first_the_first_end_model or is_first_the_second_end_model)
+
+        is_second_the_first_end_model = rel2.document.id == end_model1.document.id
+        is_second_the_second_end_model = rel2.document.id == end_model2.document.id
+        self.assertTrue(is_second_the_first_end_model or is_second_the_second_end_model)
+
+        # Test if the end model can retrieve the starter model
+        self.assertEqual(len(end_model1.starters), 1)
+
+        starter = end_model1.starters[0]
+        self.assertEqual(starter.document.id, start_model.document.id)
+
+        StartModel.destroy()
+        EndModel.destroy()
+
+
 class TransactionTestCase(ExtendedTestCase):
     def setUp(self):
 
